//
// Copyright (c) 1994-1996, David McFadzean
//
// You may distribute under the terms of the Artistic License, 
// as specified in the README file.
//
// $Header: E:/simbiosys/RCS/FSM.CPP 1.1 1996/06/26 10:54:58 dbm Exp $

#include "fsm.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	bioFSM
//

bioFSM::bioFSM (int is, int ss, int os)
//: is = # of input bits, ss = # of state bits, os = # output bits
{
	assert ((ss + is) <= 32);

    _state = 0;
    _index = 0;
    _output = 0;
    _table = 0;

    _inputSize = is;
    _stateSize = ss;
    _outputSize = os;

    _inputMask = (1 << _inputSize) - 1;
    _stateMask = (1 << _stateSize) - 1;
    _outputMask = (1 << _outputSize) - 1;
}

bioFSM::~bioFSM ()
{
    delete [_tableSize] _table;
}

void
bioFSM::Init()
{
	// table size = 2^n where n = state bits + input bits
    _tableSize = 1 << (_stateSize + _inputSize); 
    _table = new int[_tableSize]; 
    
    ASSERT (_table);

    for (int i = 0; i < _tableSize; i++)
    {
        _table[i] = rand();
    }
}       

void
bioFSM::ReadArray (int* array, int size)
{                                      
	assert (size == _tableSize);
	
    for (int i = 0; i < _tableSize; i++)
    {
    	_table[i] = array[i];
    }
}

int
bioFSM::Think (int* vector, int size)
{
    int                 i;
    unsigned int		value;

    _index = _state;

    for (i = 0; i < _inputSize; i++)
    {
    	_index = (_index<<1) | vector[i];
    }

    ASSERT (_index >= 0);
    ASSERT (_index < _tableSize);
    
    value = (unsigned int) _table[_index];
    _state = (int) ((value >> _inputSize) & _stateMask);
    _output =  (int) (value & _outputMask);

    return _output;
}

