//
// Copyright (c) 1994-1996, David McFadzean
//
// You may distribute under the terms of the Artistic License, 
// as specified in the README file.
//
// $Header: E:/simbiosys/RCS/NNET.CPP 1.1 1996/06/26 10:55:00 dbm Exp $

#include "nnet.h"

#define MAX	200

bioNNet::bioNNet (int a, int b, int c)
{
    int				i, j;

    SetSize (a, b, c);

    for (i = 0; i < _size; i++) 
    {
		for (j = 0; j < _size; j++) 
		{
		    // a _neuron cannot be connected to itself
		    if (i == j) 
		    {	
				_weights[i][j] = 0;
		    }
		    
	        // output _neurons may not be connected to each other
		    else if (_IsOutput (i) && _IsOutput (j)) 
		    {
				_weights[i][j] = 0;
		    }
		    
		    // the rest have 25% connectivity
		    else if (rand()%100 < 25) 
		    {
				_weights[i][j] = rand()%250 - 50;
		    }
		    else 
		    {
				_weights[i][j] = 0;
		    }
		}
    }
}

bioNNet::~bioNNet ()
{
    for (int i = 0; i < _size; i++) 
    {
		delete [_size] _weights[i];
    }
    delete [_size] _neuron;
    delete [_size] _newValues;
    delete [_size] _base;
    delete [_size] _weights;
}

void
bioNNet::SetSize (int inp, int inter, int out)
{
    _size = inp + inter + out;
    _inputs = inp;
    _inters = inter;
    _outputs = out;
    
    _neuron = new int[_size];
    _newValues = new int[_size];
    _base = new int[_size];
    _weights = new int*[_size];
    
    for (int i = 0; i < _size; i++) 
    {
		_weights[i] = new int[_size];
		
		// start in a random bioNNet state
		_neuron[i] = (rand()>>3)%2;
		_base[i] = MAX*(rand()>>3)%2;
    }
}

int
bioNNet::_IsOutput (int n)
{
    return (n >= (_size - _outputs));
}

void 
bioNNet::_FireNeuron (int n, int go)
{
    _newValues[n] = (go != 0);
}

int
bioNNet::Think (int* vector, int _size)
{
    int			i;
   	int			output;

	for (i = 0; i < _inputs; i++) 
	{
		_FireNeuron (i, vector[i]);
    }

    output = _Compute();
    output = _Compute();

    return output;
}

int
bioNNet::_Compute()
{
    int			i;
    int			code = 0;
    int			sum;
    
    for (i = _inputs; i < _size; i++) 
    {
		sum = _base[i];
		
		for (int j = 0; j < _size; j++) 
		{
		    sum += _neuron[j] * _weights[i][j];
		}
		
		_FireNeuron (i, (sum > 100));
    }
    
    int*		temp;

    temp = _neuron;
    _neuron = _newValues;
    _newValues = temp;

    for (i = 0; i < _outputs; i++) 
    {
		code |= _neuron[_size-1-i] << i;
    }
    
    return code;
}

void
bioNNet::Dump()
{
#if 0
    int				i, j;
    int				_inters = _size - _inputs - _outputs;
    
    printf ("input:  ");
    for (i = 0; i < _inputs; i++) 
    {
		printf ("%d ", _neuron[i]);
    } 
    printf ("\ninter:  ");
    for (i = _inputs; i < _inputs+_inters; i++) 
    {
		printf ("%d ", _neuron[i]);
    }        
    printf ("\noutput: ");
    for (i = _inputs+_inters; i < _size; i++) 
    {
		printf ("%d ", _neuron[i]);
    }
    printf ("\n");

    for (i = _inputs; i < _size; i++) 
    {
		for (j = 0; j < _size; j++) 
		{
	    	printf ("%4d ", _weights[i][j]);
		}
		printf ("\n");
    }
#endif
}

void
bioNNet::ReadArray (int* array, int count)
{
    int				i, j, k = 0;           
    int				need;
    
    need = (_size + 2) * (_size - 1);
    
    ASSERT (count == need);
    
    // Set thresholds for hidden and output units.
    for (i = _inputs; i < _size; i++) 
    {
		_base[i] = (array[k++] & 0x1) ? MAX : 0;
    }

    // Set initial activations for hidden and output units.
    for (i = _inputs; i < _size; i++) 
    {
		_neuron[i] = (array[k++] & 0x1);
    }

    // Set weights
    for (i = _inputs; i < _size; i++) 
    {
		for (j = 0; j < _size; j++) 
		{
	    	_weights[i][j] = array[k++];
		}
    }
}
